@@ -17,16 +17,18 @@
 #include "gfx2DGlue.h"
 #include "gfxContext.h"
 #include "gfxPlatform.h"
-#include "gfxPrefs.h"
+
 #include "gfxUtils.h"
 #include "mozilla/Unused.h"
 #include "mozilla/gfx/Filters.h"
 #include "mozilla/gfx/Helpers.h"
 #include "mozilla/gfx/PatternHelpers.h"
+#include "mozilla/StaticPrefs_gfx.h"
 #include "nsCSSFilterInstance.h"
 #include "nsSVGDisplayableFrame.h"
 #include "nsSVGFilterInstance.h"
 #include "nsSVGFilterPaintCallback.h"
+#include "nsSVGIntegrationUtils.h"
 #include "nsSVGUtils.h"
 
 using namespace mozilla;
@@ -35,7 +37,7 @@ using namespace mozilla::gfx;
 using namespace mozilla::image;
 
 FilterDescription nsFilterInstance::GetFilterDescription(
-    nsIContent* aFilteredElement, const nsTArray<nsStyleFilter>& aFilterChain,
+    nsIContent* aFilteredElement, Span<const StyleFilter> aFilterChain,
     bool aFilterInputIsTainted, const UserSpaceMetrics& aMetrics,
     const gfxRect& aBBox,
     nsTArray<RefPtr<SourceSurface>>& aOutAdditionalImages) {
@@ -61,7 +63,7 @@ void nsFilterInstance::PaintFilteredFrame(
     nsIFrame* aFilteredFrame, gfxContext* aCtx,
     nsSVGFilterPaintCallback* aPaintCallback, const nsRegion* aDirtyArea,
     imgDrawingParams& aImgParams, float aOpacity) {
-  auto& filterChain = aFilteredFrame->StyleEffects()->mFilters;
+  auto filterChain = aFilteredFrame->StyleEffects()->mFilters.AsSpan();
   UniquePtr<UserSpaceMetrics> metrics =
       UserSpaceMetricsForFrame(aFilteredFrame);
 
@@ -96,9 +98,8 @@ void nsFilterInstance::PaintFilteredFrame(
 }
 
 static mozilla::wr::ComponentTransferFuncType FuncTypeToWr(uint8_t aFuncType) {
+  MOZ_ASSERT(aFuncType != SVG_FECOMPONENTTRANSFER_SAME_AS_R);
   switch (aFuncType) {
-    case SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY:
-      return mozilla::wr::ComponentTransferFuncType::Identity;
     case SVG_FECOMPONENTTRANSFER_TYPE_TABLE:
       return mozilla::wr::ComponentTransferFuncType::Table;
     case SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE:
@@ -107,21 +108,22 @@ static mozilla::wr::ComponentTransferFuncType FuncTypeToWr(uint8_t aFuncType) {
       return mozilla::wr::ComponentTransferFuncType::Linear;
     case SVG_FECOMPONENTTRANSFER_TYPE_GAMMA:
       return mozilla::wr::ComponentTransferFuncType::Gamma;
+    case SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY:
     default:
-      MOZ_ASSERT(false, "unknown func type?");
+      return mozilla::wr::ComponentTransferFuncType::Identity;
   }
-  MOZ_ASSERT(false, "unknown func type?");
+  MOZ_ASSERT_UNREACHABLE("all func types not handled?");
   return mozilla::wr::ComponentTransferFuncType::Identity;
 }
 
 bool nsFilterInstance::BuildWebRenderFilters(nsIFrame* aFilteredFrame,
+                                             Span<const StyleFilter> aFilters,
                                              WrFiltersHolder& aWrFilters,
                                              Maybe<nsRect>& aPostFilterClip) {
   aWrFilters.filters.Clear();
   aWrFilters.filter_datas.Clear();
   aWrFilters.values.Clear();
 
-  auto& filterChain = aFilteredFrame->StyleEffects()->mFilters;
   UniquePtr<UserSpaceMetrics> metrics =
       UserSpaceMetricsForFrame(aFilteredFrame);
 
@@ -135,7 +137,7 @@ bool nsFilterInstance::BuildWebRenderFilters(nsIFrame* aFilteredFrame,
   // read the rendered contents of aFilteredFrame.
   bool inputIsTainted = true;
   nsFilterInstance instance(aFilteredFrame, aFilteredFrame->GetContent(),
-                            *metrics, filterChain, inputIsTainted, nullptr,
+                            *metrics, aFilters, inputIsTainted, nullptr,
                             scaleMatrixInDevUnits, nullptr, nullptr, nullptr,
                             nullptr);
 
@@ -146,7 +148,7 @@ bool nsFilterInstance::BuildWebRenderFilters(nsIFrame* aFilteredFrame,
   // If there are too many filters to render, then just pretend that we
   // succeeded, and don't render any of them.
   if (instance.mFilterDescription.mPrimitives.Length() >
-      gfxPrefs::WebRenderMaxFilterOpsPerChain()) {
+      StaticPrefs::gfx_webrender_max_filter_ops_per_chain()) {
     return true;
   }
 
@@ -202,33 +204,18 @@ bool nsFilterInstance::BuildWebRenderFilters(nsIFrame* aFilteredFrame,
           attr.as<ColorMatrixAttributes>();
 
       float transposed[20];
-      if (!gfx::ComputeColorMatrix(attributes, transposed)) {
+      if (gfx::ComputeColorMatrix(attributes, transposed)) {
+        float matrix[20] = {
+            transposed[0], transposed[5], transposed[10], transposed[15],
+            transposed[1], transposed[6], transposed[11], transposed[16],
+            transposed[2], transposed[7], transposed[12], transposed[17],
+            transposed[3], transposed[8], transposed[13], transposed[18],
+            transposed[4], transposed[9], transposed[14], transposed[19]};
+
+        aWrFilters.filters.AppendElement(wr::FilterOp::ColorMatrix(matrix));
+      } else {
         filterIsNoop = true;
-        continue;
       }
-
-      auto almostEq = [](float a, float b) -> bool {
-        return fabs(a - b) < 0.00001;
-      };
-
-      if (!almostEq(transposed[15], 0.0) || !almostEq(transposed[16], 0.0) ||
-          !almostEq(transposed[17], 0.0) || !almostEq(transposed[18], 1.0) ||
-          !almostEq(transposed[3], 0.0) || !almostEq(transposed[8], 0.0) ||
-          !almostEq(transposed[13], 0.0)) {
-        // WebRender currently pretends to take the full 4x5 matrix but discards
-        // the components related to alpha. So bail out in this case until
-        // it is fixed.
-        return false;
-      }
-
-      float matrix[20] = {
-          transposed[0], transposed[5], transposed[10], transposed[15],
-          transposed[1], transposed[6], transposed[11], transposed[16],
-          transposed[2], transposed[7], transposed[12], transposed[17],
-          transposed[3], transposed[8], transposed[13], transposed[18],
-          transposed[4], transposed[9], transposed[14], transposed[19]};
-
-      aWrFilters.filters.AppendElement(wr::FilterOp::ColorMatrix(matrix));
     } else if (attr.is<GaussianBlurAttributes>()) {
       if (finalClip) {
         // There's a clip that needs to apply before the blur filter, but
@@ -264,14 +251,14 @@ bool nsFilterInstance::BuildWebRenderFilters(nsIFrame* aFilteredFrame,
         return false;
       }
 
-      Color color = shadow.mColor;
+      sRGBColor color = shadow.mColor;
       if (!primNeedsSrgb) {
-        color = Color(gsRGBToLinearRGBMap[uint8_t(color.r * 255)],
-                      gsRGBToLinearRGBMap[uint8_t(color.g * 255)],
-                      gsRGBToLinearRGBMap[uint8_t(color.b * 255)], color.a);
+        color = sRGBColor(gsRGBToLinearRGBMap[uint8_t(color.r * 255)],
+                          gsRGBToLinearRGBMap[uint8_t(color.g * 255)],
+                          gsRGBToLinearRGBMap[uint8_t(color.b * 255)], color.a);
       }
       wr::Shadow wrShadow;
-      wrShadow.offset = {(float)shadow.mOffset.x, (float)shadow.mOffset.y};
+      wrShadow.offset = {shadow.mOffset.x, shadow.mOffset.y};
       wrShadow.color = wr::ToColorF(ToDeviceColor(color));
       wrShadow.blur_radius = stdDev.width;
       wr::FilterOp filterOp = wr::FilterOp::DropShadow(wrShadow);
@@ -302,15 +289,19 @@ bool nsFilterInstance::BuildWebRenderFilters(nsIFrame* aFilteredFrame,
       values->AppendElements(attributes.mValues[0]);
       filterData.R_values_count = attributes.mValues[0].Length();
 
-      filterData.funcG_type = FuncTypeToWr(attributes.mTypes[1]);
+      size_t indexToUse =
+          attributes.mTypes[1] == SVG_FECOMPONENTTRANSFER_SAME_AS_R ? 0 : 1;
+      filterData.funcG_type = FuncTypeToWr(attributes.mTypes[indexToUse]);
       size_t G_startindex = values->Length();
-      values->AppendElements(attributes.mValues[1]);
-      filterData.G_values_count = attributes.mValues[1].Length();
+      values->AppendElements(attributes.mValues[indexToUse]);
+      filterData.G_values_count = attributes.mValues[indexToUse].Length();
 
-      filterData.funcB_type = FuncTypeToWr(attributes.mTypes[2]);
+      indexToUse =
+          attributes.mTypes[2] == SVG_FECOMPONENTTRANSFER_SAME_AS_R ? 0 : 2;
+      filterData.funcB_type = FuncTypeToWr(attributes.mTypes[indexToUse]);
       size_t B_startindex = values->Length();
-      values->AppendElements(attributes.mValues[2]);
-      filterData.B_values_count = attributes.mValues[2].Length();
+      values->AppendElements(attributes.mValues[indexToUse]);
+      filterData.B_values_count = attributes.mValues[indexToUse].Length();
 
       filterData.funcA_type = FuncTypeToWr(attributes.mTypes[3]);
       size_t A_startindex = values->Length();
@@ -375,7 +366,7 @@ nsRegion nsFilterInstance::GetPostFilterDirtyArea(
   }
 
   gfxMatrix tm = nsSVGUtils::GetCanvasTM(aFilteredFrame);
-  auto& filterChain = aFilteredFrame->StyleEffects()->mFilters;
+  auto filterChain = aFilteredFrame->StyleEffects()->mFilters.AsSpan();
   UniquePtr<UserSpaceMetrics> metrics =
       UserSpaceMetricsForFrame(aFilteredFrame);
   // Hardcode InputIsTainted to true because we don't want JS to be able to
@@ -396,7 +387,7 @@ nsRegion nsFilterInstance::GetPostFilterDirtyArea(
 nsRegion nsFilterInstance::GetPreFilterNeededArea(
     nsIFrame* aFilteredFrame, const nsRegion& aPostFilterDirtyRegion) {
   gfxMatrix tm = nsSVGUtils::GetCanvasTM(aFilteredFrame);
-  auto& filterChain = aFilteredFrame->StyleEffects()->mFilters;
+  auto filterChain = aFilteredFrame->StyleEffects()->mFilters.AsSpan();
   UniquePtr<UserSpaceMetrics> metrics =
       UserSpaceMetricsForFrame(aFilteredFrame);
   // Hardcode InputIsTainted to true because we don't want JS to be able to
@@ -428,7 +419,7 @@ nsRect nsFilterInstance::GetPostFilterBounds(nsIFrame* aFilteredFrame,
   }
 
   gfxMatrix tm = nsSVGUtils::GetCanvasTM(aFilteredFrame);
-  auto& filterChain = aFilteredFrame->StyleEffects()->mFilters;
+  auto filterChain = aFilteredFrame->StyleEffects()->mFilters.AsSpan();
   UniquePtr<UserSpaceMetrics> metrics =
       UserSpaceMetricsForFrame(aFilteredFrame);
   // Hardcode InputIsTainted to true because we don't want JS to be able to
@@ -446,10 +437,9 @@ nsRect nsFilterInstance::GetPostFilterBounds(nsIFrame* aFilteredFrame,
 
 nsFilterInstance::nsFilterInstance(
     nsIFrame* aTargetFrame, nsIContent* aTargetContent,
-    const UserSpaceMetrics& aMetrics,
-    const nsTArray<nsStyleFilter>& aFilterChain, bool aFilterInputIsTainted,
-    nsSVGFilterPaintCallback* aPaintCallback, const gfxMatrix& aPaintTransform,
-    const nsRegion* aPostFilterDirtyRegion,
+    const UserSpaceMetrics& aMetrics, Span<const StyleFilter> aFilterChain,
+    bool aFilterInputIsTainted, nsSVGFilterPaintCallback* aPaintCallback,
+    const gfxMatrix& aPaintTransform, const nsRegion* aPostFilterDirtyRegion,
     const nsRegion* aPreFilterDirtyRegion,
     const nsRect* aPreFilterVisualOverflowRectOverride,
     const gfxRect* aOverrideBBox)
@@ -555,9 +545,9 @@ gfxRect nsFilterInstance::FilterSpaceToUserSpace(
   return userSpaceRect;
 }
 
-nsresult nsFilterInstance::BuildPrimitives(
-    const nsTArray<nsStyleFilter>& aFilterChain, nsIFrame* aTargetFrame,
-    bool aFilterInputIsTainted) {
+nsresult nsFilterInstance::BuildPrimitives(Span<const StyleFilter> aFilterChain,
+                                           nsIFrame* aTargetFrame,
+                                           bool aFilterInputIsTainted) {
   nsTArray<FilterPrimitiveDescription> primitiveDescriptions;
 
   for (uint32_t i = 0; i < aFilterChain.Length(); i++) {
@@ -577,13 +567,13 @@ nsresult nsFilterInstance::BuildPrimitives(
 }
 
 nsresult nsFilterInstance::BuildPrimitivesForFilter(
-    const nsStyleFilter& aFilter, nsIFrame* aTargetFrame, bool aInputIsTainted,
+    const StyleFilter& aFilter, nsIFrame* aTargetFrame, bool aInputIsTainted,
     nsTArray<FilterPrimitiveDescription>& aPrimitiveDescriptions) {
   NS_ASSERTION(mUserSpaceToFilterSpaceScale.width > 0.0f &&
                    mFilterSpaceToUserSpaceScale.height > 0.0f,
                "scale factors between spaces should be positive values");
 
-  if (aFilter.GetType() == NS_STYLE_FILTER_URL) {
+  if (aFilter.IsUrl()) {
     // Build primitives for an SVG filter.
     nsSVGFilterInstance svgFilterInstance(aFilter, aTargetFrame, mTargetContent,
                                           mMetrics, mTargetBBox,
@@ -601,7 +591,7 @@ nsresult nsFilterInstance::BuildPrimitivesForFilter(
   // If we don't have a frame, use opaque black for shadows with unspecified
   // shadow colors.
   nscolor shadowFallbackColor =
-      mTargetFrame ? mTargetFrame->StyleColor()->mColor.ToColor()
+      mTargetFrame ? mTargetFrame->StyleText()->mColor.ToColor()
                    : NS_RGB(0, 0, 0);
 
   nsCSSFilterInstance cssFilterInstance(
@@ -768,6 +758,11 @@ void nsFilterInstance::Render(gfxContext* aCtx, imgDrawingParams& aImgParams,
   Rect renderRect = IntRectToRect(filterRect);
   RefPtr<DrawTarget> dt = aCtx->GetDrawTarget();
 
+  MOZ_ASSERT(dt);
+  if (!dt->IsValid()) {
+    return;
+  }
+
   BuildSourcePaints(aImgParams);
   RefPtr<FilterNode> sourceGraphic, fillPaint, strokePaint;
   if (mFillPaint.mSourceSurface) {
@@ -791,7 +786,7 @@ void nsFilterInstance::Render(gfxContext* aCtx, imgDrawingParams& aImgParams,
   }
 
   RefPtr<FilterNode> resultFilter = FilterNodeGraphFromDescription(
-      aCtx->GetDrawTarget(), mFilterDescription, renderRect, sourceGraphic,
+      dt, mFilterDescription, renderRect, sourceGraphic,
       mSourceGraphic.mSurfaceRect, fillPaint, strokePaint, mInputImages);
 
   if (!resultFilter) {
@@ -799,8 +794,7 @@ void nsFilterInstance::Render(gfxContext* aCtx, imgDrawingParams& aImgParams,
     return;
   }
 
-  BuildSourceImage(aCtx->GetDrawTarget(), aImgParams, resultFilter,
-                   sourceGraphic, renderRect);
+  BuildSourceImage(dt, aImgParams, resultFilter, sourceGraphic, renderRect);
   if (sourceGraphic) {
     if (mSourceGraphic.mSourceSurface) {
       sourceGraphic->SetInput(IN_TRANSFORM_IN, mSourceGraphic.mSourceSurface);
@@ -810,8 +804,7 @@ void nsFilterInstance::Render(gfxContext* aCtx, imgDrawingParams& aImgParams,
     }
   }
 
-  aCtx->GetDrawTarget()->DrawFilter(resultFilter, renderRect, Point(0, 0),
-                                    DrawOptions(aOpacity));
+  dt->DrawFilter(resultFilter, renderRect, Point(0, 0), DrawOptions(aOpacity));
 }
 
 nsRegion nsFilterInstance::ComputePostFilterDirtyRegion() {
